@@ -1,7 +1,7 @@
 ;;; -*- Mode: LISP; Syntax: COMMON-LISP; Package: TBNL; Base: 10 -*-
-;;; $Header: /usr/local/cvsrep/tbnl/modlisp.lisp,v 1.49 2005/11/29 09:01:05 edi Exp $
+;;; $Header: /usr/local/cvsrep/tbnl/modlisp.lisp,v 1.63 2006/04/25 19:57:40 edi Exp $
 
-;;; Copyright (c) 2004-2005, Dr. Edmund Weitz.  All rights reserved.
+;;; Copyright (c) 2004-2006, Dr. Edmund Weitz.  All rights reserved.
 
 ;;; Redistribution and use in source and binary forms, with or without
 ;;; modification, are permitted provided that the following conditions
@@ -51,61 +51,66 @@ to the mod_lisp socket stream"
 to the http/Araneida socket stream"
   (format *tbnl-stream*
           (cond ((string= "Status" key)
-                 (load-time-value (formatter "HTTP/1.0 ~*~A")))
+                 (load-time-value (formatter "HTTP/1.1 ~*~A")))
                 (t (load-time-value (formatter "~A: ~A"))))
           key
           (cl-ppcre:regex-replace-all "[\\r\\n]" value " "))
   (write-char #\Return *tbnl-stream*)
   (write-char #\NewLine *tbnl-stream*))
 
-(defun send-output (content)
-  "Sends all headers and the content body to *TBNL-STREAM*.  Handles
-the supported return codes accordingly.  Called by
-PROCESS-REQUEST."
+(defun start-output (&optional content)
+  "Sends all headers and maybe the content body to *TBNL-STREAM*.
+Handles the supported return codes accordingly.  Called by
+PROCESS-REQUEST.  Returns the stream to write to."
+  (declare (notinline address-string))
+  (when *headers-sent*
+    (return-from start-output))
+  (setq *headers-sent* t)
   (let* ((return-code (return-code *reply*))
          (status-line (status-line return-code))
          (request-method (request-method))
-         (head-request-p (eq request-method :head)))
+         (head-request-p (eq request-method :head))
+         content-modified-p)
     (unless status-line
       (setq content (escape-for-html
                      (format nil "Unknown http return code: ~A" return-code))
+            content-modified-p t
             return-code +http-internal-server-error+
             status-line (status-line return-code)))
     (unless (member return-code '(#.+http-ok+ #.+http-not-modified+))
       ;; call error handler, if any - should return NIL if it can't
       ;; handle the error.
-      (when *http-error-handler*
-        (setq content (or (funcall *http-error-handler* return-code)
-                          content)))
-      ;; handle common return codes other than 200, which weren't
-      ;; handled by the error handler.
-      (unless content
-        (setf (content-type *reply*)
-              "text/html; charset=iso-8859-1"
-              content
-              (format nil "<html><head><title>~D ~A</title></head><body><h1>~A</h1>~A<p><hr><address>~A<a href='http://weitz.de/tbnl/'>TBNL ~A</a> <a href='~A'>(~A ~A)</a> at ~A Port ~D</address></p></body></html>"
-                      return-code status-line status-line
-                      (case return-code
-                        ((#.+http-internal-server-error+)
-                         content)
-                        ((#.+http-moved-temporarily+ #.+http-moved-permanently+)
-                         (format nil "The document has moved <a href='~A'>here</a>"
-                                 (header-out "Location")))
-                        ((#.+http-authorization-required+)
-                         "The server could not verify that you are authorized to access the document requested. Either you supplied the wrong credentials \(e.g., bad password), or your browser doesn't understand how to supply the credentials required.")
-                        ((#.+http-forbidden+)
-                         (format nil "You don't have permission to access ~A on this server."
-                                 (script-name)))
-                        ((#.+http-not-found+)
-                         (format nil "The requested URL ~A was not found on this server."
-                                 (script-name))))
-                      (tbnl-info-string)
-                      *tbnl-version*
-                      +implementation-link+
-                      (lisp-implementation-type)
-                      (lisp-implementation-version)
-                      (host *request*)
-                      (server-port :request *request*)))))
+      (let (error-handled-p)
+        (when *http-error-handler*
+          (setq error-handled-p (funcall *http-error-handler* return-code)
+                content (or error-handled-p content)
+                content-modified-p (or content-modified-p error-handled-p)))
+        ;; handle common return codes other than 200, which weren't
+        ;; handled by the error handler.
+        (unless error-handled-p
+          (setf (content-type *reply*)
+                  "text/html; charset=iso-8859-1"
+                content-modified-p t
+                content
+                  (format nil "<html><head><title>~D ~A</title></head><body><h1>~A</h1>~A<p><hr>~A</p></body></html>"
+                          return-code status-line status-line
+                          (case return-code
+                            ((#.+http-internal-server-error+)
+                             content)
+                            ((#.+http-moved-temporarily+ #.+http-moved-permanently+)
+                             (format nil "The document has moved <a href='~A'>here</a>"
+                                     (header-out "Location")))
+                            ((#.+http-authorization-required+)
+                             "The server could not verify that you are authorized to access the document requested. Either you supplied the wrong credentials \(e.g., bad password), or your browser doesn't understand how to supply the credentials required.")
+                            ((#.+http-forbidden+)
+                             (format nil "You don't have permission to access ~A on this server."
+                                     (script-name)))
+                            ((#.+http-not-found+)
+                             (format nil "The requested URL ~A was not found on this server."
+                                     (script-name)))
+                            ((#.+http-bad-request+)
+                             "Your browser sent a request that this server could not understand."))
+                          (address-string))))))
     ;; access log message
     (when (and *show-access-log-messages*
                (not *use-apache-log*))
@@ -116,29 +121,31 @@ PROCESS-REQUEST."
                      (referer) (user-agent))))
     ;; start with status line
     (write-header-line "Status" (format nil "~D ~A" return-code status-line))
-    ;; if there's content write the corresponding headers
-    (when content
-      (let ((content-length (or (content-length *reply*)
-                                (length content))))
-        (when (plusp content-length)
-          (when (starts-with-one-of-p (content-type *reply*)
-                                      *content-types-for-url-rewrite*)
-            ;; if the Content-Type header starts with one of the strings
-            ;; in *CONTENT-TYPES-FOR-URL-REWRITE* then maybe rewrite the
-            ;; content
-            (setq content
-                    (maybe-rewrite-urls-for-session content)
-                  content-length
-                    (or (content-length *reply*)
-                        (length content))))
-          (write-header-line "Content-Length" (format nil "~D" content-length))
-          (when *use-modlisp-headers*
-            (write-header-line "Lisp-Content-Length"
-                               (cond (head-request-p "0")
-                                     (t (format nil "~D" content-length))))
-            (write-header-line "Keep-Socket" "1")
-            (setq *close-tbnl-stream* nil))
-          (write-header-line "Content-Type" (content-type *reply*)))))
+    (let ((content-length (or (and (not content-modified-p)
+                                   (content-length *reply*))
+                              (length content))))
+      (when (and content (plusp content-length))
+        (when (starts-with-one-of-p (content-type *reply*)
+                                    *content-types-for-url-rewrite*)
+          ;; if the Content-Type header starts with one of the strings
+          ;; in *CONTENT-TYPES-FOR-URL-REWRITE* then maybe rewrite the
+          ;; content
+          (setq content
+                (maybe-rewrite-urls-for-session content)
+                content-length
+                (or (and (not content-modified-p)
+                         (content-length *reply*))
+                    (length content)))))
+      ;; write the corresponding headers for the content
+      (when (and content content-length)
+        (write-header-line "Content-Length" (format nil "~D" content-length))
+        (when *use-modlisp-headers*
+          (write-header-line "Lisp-Content-Length"
+                             (cond (head-request-p "0")
+                                   (t (format nil "~D" content-length))))
+          (write-header-line "Keep-Socket" "1")
+          (setq *close-tbnl-stream* nil))))
+    (write-header-line "Content-Type" (content-type *reply*))
     ;; write all headers from the REPLY object
     (loop for (key . value) in (headers-out *reply*)
           do (write-header-line key value))
@@ -165,7 +172,6 @@ PROCESS-REQUEST."
           (t
            (write-char #\Return *tbnl-stream*)))
     (write-char #\NewLine *tbnl-stream*)
-
     ;; now optional content
     (cond ((or (null content)
                head-request-p)
@@ -178,7 +184,19 @@ PROCESS-REQUEST."
            (ignore-errors
              (write-sequence content *tbnl-stream*)))
           (t
-           nil))))
+           nil))
+    *tbnl-stream*))
+
+(defun send-headers ()
+  "Sends the initial status line and all headers as determined by
+the REPLY object *REPLY*.  Returns a stream to which the body of
+the reply can be written.  Once this function has been called
+further changes to *REPLY* don't have any effect.  Also,
+automatic handling of errors \(i.e. sending the corresponding
+status code to the browser, etc.) is turned off for this request.
+If your handlers return the full body as a string or as an array
+of octets you should NOT call this function."
+  (start-output))
 
 (defun no-cache ()
   "Adds appropriate headers to completely prevent caching on most browsers."
@@ -230,10 +248,10 @@ request is sent to the browser, otherwise a 302."
   "Processes COMMAND as created by GET-REQUEST-DATA using the
 corresponding user funtion from *DISPATCH-TABLE*.  Sets up REPLY,
 REQUEST, and SESSION objects.  Called by LISTEN-FOR-REQUEST."
-  (let (*tmp-files*)
+  (let (*tmp-files* *headers-sent*)
     (unwind-protect
         (let* ((*session* nil)
-               ;; first create a REPLY object so we can immedetialy start
+               ;; first create a REPLY object so we can immediately start
                ;; logging
                (*reply* (debug-value *reply* (make-instance 'reply)))
                (*request* (debug-value *request*
@@ -259,6 +277,11 @@ REQUEST, and SESSION objects.  Called by LISTEN-FOR-REQUEST."
                                                  cond
                                                  *log-lisp-backtraces-p*
                                                  backtrace))
+                                  ;; if the headers were already sent
+                                  ;; the error happens within the body
+                                  ;; and we have to close the stream
+                                  (when *headers-sent*
+                                    (setq *close-tbnl-stream* t))
                                   (throw 'tbnl-handler-done
                                     (values nil cond))))
                                (warning
@@ -270,11 +293,16 @@ REQUEST, and SESSION objects.  Called by LISTEN-FOR-REQUEST."
                                                  cond
                                                  *log-lisp-backtraces-p*
                                                  backtrace)))))
-                  (dispatch-request *dispatch-table*)))
+                  ;; skip dispatch if bad request
+                  (when (eq (return-code) +http-ok+)
+                    ;; read post data to clear stream
+                    (raw-post-data)
+                    ;; now do the work
+                    (dispatch-request *dispatch-table*))))
             (when error
               (setf (return-code *reply*)
                       +http-internal-server-error+))
-            (send-output
+            (start-output
              (debug-value *body*
                           (cond ((and error *show-lisp-errors-p*)
                                  (format nil "<pre>~A~:[~*~;~%~%~A~]</pre>"
@@ -283,7 +311,8 @@ REQUEST, and SESSION objects.  Called by LISTEN-FOR-REQUEST."
                                          (escape-for-html (format nil "~A" backtrace))))
                                 (error
                                  "An error has occured")
-                                (t body))))))
+                                (t body)))))
+          t)
       (loop for path in *tmp-files*
             when (and (pathnamep path)
                       (probe-file path))
@@ -327,9 +356,12 @@ converts them into an alist."
                "Adds the cons PAIR to the list HEADERS of headers
 which are already there.  Takes care of multiple headers with the same
 key."
-               (let ((existing-header (string-assoc (car pair) headers)))
+               (let ((existing-header (assoc (car pair) headers :test #'string-equal)))
                  (cond (existing-header
-                        (rplacd (string-assoc (car pair) headers) (cdr pair)))
+                        (setf (cdr existing-header)
+                              (format nil "~A ~A"
+                                      (cdr existing-header)
+                                      (cdr pair))))
                        (t (push pair headers))))))
       (loop
        (let ((pair (split-header (read-header-line))))
@@ -351,10 +383,11 @@ and in FIRST-LINE.  Returns an alist of the headers."
             headers)
       (push (cons "url" url-string)
             headers)
-      (when protocol
-        (push (cons "server-protocol"
-                    (string-trim '(#\Space #\NewLine #\Return) protocol))
-              headers))
+      (unless protocol
+        (setq protocol "HTTP/0.9"))
+      (push (cons "server-protocol"
+                  (string-trim '(#\Space #\NewLine #\Return) protocol))
+            headers)
       (push (cons "content-stream" *tbnl-stream*) headers)
       headers)))
 
@@ -372,7 +405,15 @@ the results as an alist."
              ;; come from mod_lisp so we assume Araneida or no front-end
              ;; at all
              (setq *use-modlisp-headers* nil)
-             (read-http-request first-line))
+             (let ((headers (read-http-request first-line)))
+               (when (equalp (string-assoc "Expect" headers) "100-continue")
+                 (write-header-line "Status" (format nil "~D ~A"
+                                                     +http-continue+
+                                                     (status-line +http-continue+)))
+                 (write-char #\Return *tbnl-stream*)
+                 (write-char #\Newline *tbnl-stream*)
+                 (force-output *tbnl-stream*))
+               headers))
             ;; now we assume mod_lisp, so we read alternating
             ;; key/value lines
             (t (setq *use-modlisp-headers* t)
@@ -391,6 +432,7 @@ the results as an alist."
                          headers))
                  headers))))))
 
+#-:hunchentoot
 (defun listen-for-request (*tbnl-stream* command-processor &rest args)
   "Listens on *TBNL-STREAM* for an incoming request.  Packages the
 command using GET-REQUEST-DATA and passes it to the COMMAND-PROCESSOR
@@ -401,8 +443,7 @@ be called by a KMRCL:LISTENER object."
       (loop for *close-tbnl-stream* = t
             for *use-modlisp-headers* = nil
             for *tbnl-socket-usage-counter* from 0
-            for command = (debug-value *command*
-                                       (ignore-errors (get-request-data)))
+            for command = (debug-value *command* (get-request-data))
             while command
             do (cond ((ignore-errors
                         (funcall command-processor command))
@@ -418,6 +459,7 @@ be called by a KMRCL:LISTENER object."
     (ignore-errors
       (kmrcl:close-active-socket *tbnl-stream*))))
 
+#-:hunchentoot
 (defun start-tbnl ()
   "Starts listening on port *TBNL-PORT* if needed.  Initializes
 *SESSION-SECRET* if needed.  Returns the newly created or already
@@ -441,6 +483,7 @@ existing KMRCL:LISTENER object."
                                           :remote-host-checker nil))))
   (kmrcl:init/listener *listener* :start))
 
+#-:hunchentoot
 (defun stop-tbnl ()
   "Stops the KMRCL:LISTENER object bound to *LISTENER* if it exists."
   (cond ((and (boundp '*listener*)
