@@ -1,7 +1,7 @@
 ;;; -*- Mode: LISP; Syntax: COMMON-LISP; Package: TBNL; Base: 10 -*-
-;;; $Header: /usr/local/cvsrep/tbnl/request.lisp,v 1.27 2005/11/17 00:01:06 edi Exp $
+;;; $Header: /usr/local/cvsrep/tbnl/request.lisp,v 1.36 2006/05/12 09:27:44 edi Exp $
 
-;;; Copyright (c) 2004-2005, Dr. Edmund Weitz.  All rights reserved.
+;;; Copyright (c) 2004-2006, Dr. Edmund Weitz.  All rights reserved.
 
 ;;; Redistribution and use in source and binary forms, with or without
 ;;; modification, are permitted provided that the following conditions
@@ -83,8 +83,19 @@ supposed to be of content type 'multipart/form-data'."
                                     (rfc2388:content-type part :as-string t))
                               contents))))))
 
+(defun get-post-data (&optional (request *request*))
+  "Reads the posted data from the stream and stores the raw contents
+in the corresponding slot of the REQUEST object."
+  (let* ((headers-in (headers-in request))
+         (content-length (string-assoc "content-length" headers-in)))
+    (when content-length
+      (let ((content (make-string (parse-integer content-length
+                                                 :junk-allowed t))))
+        (read-sequence content (string-assoc "content-stream" headers-in))
+        (setf (slot-value request 'raw-post-data) content)))))
+
 (defmethod initialize-instance :after ((request request) &rest init-args)
-  "The only initarg for a REQUEST object is :HEADERS-IN. Also other
+  "The only initarg for a REQUEST object is :HEADERS-IN. All other
 slot values are computed in this :AFTER method."
   (declare (ignore init-args))
   (with-slots ((headers-in headers-in)
@@ -94,52 +105,59 @@ slot values are computed in this :AFTER method."
                (script-name script-name)
                (query-string query-string)
                (session session))
-      request
-    ;; compute SCRIPT-NAME and QUERY-STRING slots from REQUEST_URI
-    ;; environment variable
-    (let* ((uri (request-uri request))
-           (match-start (position #\? uri)))
-      (cond
-        (match-start
-          (setq script-name (subseq uri 0 match-start)
-                query-string (subseq uri (1+ match-start))))
-        (t (setq script-name uri))))
-    ;; some clients send requests like "GET http://server/foo.html
-    ;; HTTP/1.0"...
-    (setq script-name (cl-ppcre:regex-replace "^https?://[^/]+" script-name ""))
-    ;; if the content-type is 'application/x-www-form-urlencoded' or
-    ;; 'multipart/form-data' compute the post parameters from the
-    ;; content body
-    (let ((content-type (string-assoc "content-type" headers-in))
-          (content-length (string-assoc "content-length" headers-in))
-          (content-stream (string-assoc "content-stream" headers-in)))
-      (setq post-parameters
-              (cond ((string-equal "application/x-www-form-urlencoded" content-type)
-                     (and content-length
-                          (form-url-encoded-list-to-alist
-                           (cl-ppcre:split "&"
-                                           (let ((content (make-string (parse-integer content-length
-                                                                                      :junk-allowed t))))
-                                             (read-sequence content content-stream)
-                                             (setf (slot-value request 'raw-post-data) content)
-                                             content)))))
-                    ((starts-with-p content-type "multipart/form-data;" :test #'char-equal)
-                     (handler-case
-                       (parse-rfc2388-form-data content-stream content-type)
-                       (error (msg)
-                         (log-message :error "While parsing multipart/form-data parameters: ~A" msg)
-                         nil))))))
-    ;; compute GET parameters from query string and cookies from the
-    ;; incoming 'Cookie' header
-    (setq get-parameters
-            (form-url-encoded-list-to-alist
-             (cl-ppcre:split "&" query-string))
-          cookies-in
-            (form-url-encoded-list-to-alist
-             (cl-ppcre:split ";"
-                             (string-assoc "Cookie" headers-in)))
-          session (session-verify request)
-          *session* session)))
+              request
+     (handler-case
+       (progn
+         ;; compute SCRIPT-NAME and QUERY-STRING slots from
+         ;; REQUEST_URI environment variable
+         (let* ((uri (request-uri request))
+                (match-start (position #\? uri)))
+           (cond
+             (match-start
+              (setq script-name (subseq uri 0 match-start)
+                    query-string (subseq uri (1+ match-start))))
+             (t (setq script-name uri))))
+         ;; some clients send requests like
+         ;; "GET http://server/foo.html HTTP/1.0"...
+         (setq script-name (cl-ppcre:regex-replace "^https?://[^/]+" script-name ""))
+         ;; if the content-type is 'application/x-www-form-urlencoded'
+         ;; or 'multipart/form-data', compute the post parameters from
+         ;; the content body
+         (let ((content-type (string-assoc "content-type" headers-in)))
+           (setq post-parameters
+                   (cond ((starts-with-p content-type "application/x-www-form-urlencoded"
+                                         :test #'char-equal)
+                          (and (string-assoc "content-length" headers-in)
+                               (form-url-encoded-list-to-alist
+                                (cl-ppcre:split "&"
+                                                (get-post-data request)))))
+                         ((starts-with-p content-type "multipart/form-data;"
+                                         :test #'char-equal)
+                          (setf (slot-value request 'raw-post-data) t)
+                          (handler-case
+                            (parse-rfc2388-form-data (string-assoc "content-stream"
+                                                                   headers-in)
+                                                     content-type)
+                            (error (msg)
+                              (log-message :error
+                                           "While parsing multipart/form-data parameters: ~A"
+                                           msg)
+                              nil))))))
+         ;; compute GET parameters from query string and cookies from
+         ;; the incoming 'Cookie' header
+         (setq get-parameters
+                 (form-url-encoded-list-to-alist
+                  (cl-ppcre:split "&" query-string))
+               cookies-in
+                 (form-url-encoded-list-to-alist
+                  (cl-ppcre:split ";"
+                                  (string-assoc "Cookie" headers-in)))
+               session (session-verify request)
+               *session* session))
+       (error (cond)
+         (log-message* "Error when creating REQUEST object: ~A" cond)
+         ;; we assume it's not our fault...
+         (setf (return-code) +http-bad-request+)))))
 
 (defun recompute-request-parameters (&key (request *request*) (external-format *tbnl-default-external-format*))
   "Recomputes the GET and POST parameters, and the incoming cookies
@@ -157,6 +175,9 @@ a different external format."
                                             external-format)
           post-parameters
             (and raw-post-data
+                 (starts-with-p (string-assoc "content-type" headers-in)
+                                "application/x-www-form-urlencoded"
+                                :test #'char-equal)
                  (form-url-encoded-list-to-alist (cl-ppcre:split "&" raw-post-data)
                                                  external-format))
           cookies-in
@@ -346,7 +367,8 @@ string is compared to the one generated from the supplied TIME."
 
 (defun raw-post-data (&optional (request *request*))
   "Returns the raw string sent as the body of a POST request."
-(slot-value request 'raw-post-data))
+  (or (slot-value request 'raw-post-data)
+      (get-post-data request)))
 
 (defun aux-request-value (symbol &optional (request *request*))
   "Returns the value associated with SYMBOL from the request object
@@ -383,4 +405,3 @@ REQUEST."
             (delete symbol (aux-data request)
                     :key #'car :test #'eq)))
   (values))
-
