@@ -1,7 +1,7 @@
 ;;; -*- Mode: LISP; Syntax: COMMON-LISP; Package: TBNL; Base: 10 -*-
-;;; $Header: /usr/local/cvsrep/tbnl/test/test.lisp,v 1.27 2005/09/23 23:33:42 edi Exp $
+;;; $Header: /usr/local/cvsrep/tbnl/test/test.lisp,v 1.31 2006-01-03 18:37:56 edi Exp $
 
-;;; Copyright (c) 2004-2005, Dr. Edmund Weitz.  All rights reserved.
+;;; Copyright (c) 2004-2006, Dr. Edmund Weitz.  All rights reserved.
 
 ;;; Redistribution and use in source and binary forms, with or without
 ;;; modification, are permitted provided that the following conditions
@@ -349,16 +349,13 @@ in "
 (defun send-file ()
   (let* ((path (get-parameter "path"))
          (file-info (and path
-                         (find (pathname path) *tmp-test-files* :key #'first :test #'equal))))
+                         (find (pathname path) *tmp-test-files*
+                               :key #'first :test #'equal))))
     (unless file-info
       (setf (return-code *reply*)
               +http-not-found+)
       (return-from send-file))
-    ;; ugly hack just for this demo
-    (let ((dispatcher
-            (create-static-file-dispatcher-and-handler
-             (script-name) path (third file-info))))
-      (funcall (funcall dispatcher *request*)))))
+    (handle-static-file path (third file-info))))
 
 (defparameter *headline*
   (load-time-value              
@@ -389,6 +386,21 @@ in "
                      (merge-pathnames "UTF-8-demo.html"
                                       *load-truename*))))
 
+
+#+:tbnl-bivalent-streams
+(defun stream-direct ()
+  (setf (content-type)
+          "text/html; charset=utf-8")
+  (let ((stream (send-headers))
+        (buffer (make-array 1024 :element-type '(unsigned-byte 8))))
+    (with-open-file (in (load-time-value
+                         (merge-pathnames "UTF-8-demo.html"
+                                          *load-truename*))
+                        :element-type '(unsigned-byte 8))
+      (loop for pos = (read-sequence buffer in)
+            until (zerop pos) 
+            do (write-sequence buffer stream :end pos)))))
+
 (defun menu ()
   (with-html
     (:html
@@ -416,6 +428,8 @@ in "
               (:tr (:td (:a :href "/tbnl/test/authorization.html"
                             "Authorization")
                         " (user 'nanook', password 'igloo')"))
+              (:tr (:td (:a :href "/tbnl/code/test.lisp"
+                            "The source code of this test")))
               (:tr (:td (:a :href "/tbnl/test/image.jpg"
                             "Binary data, delivered from file")
                         " \(a picture)"))
@@ -425,7 +439,11 @@ in "
               #+:lispworks
               (:tr (:td (:a :href "/tbnl/test/utf-8.html"
                             "UTF-8 demo")
-                        " \(currently Lispworks-only)"))
+                    " \(currently Lispworks-only)"))
+              #+:tbnl-bivalent-streams
+              (:tr (:td (:a :href "/tbnl/test/umlaut.txt"
+                            "UTF-8 demo")
+                    " \(writing directly to the stream)"))
               (:tr (:td (:a :href "/tbnl/test/upload.html"
                             "File Uploads")))
               (:tr (:td (:a :href "/tbnl/test/forbidden.html"
@@ -447,30 +465,37 @@ in "
                             (:code "*DEFAULT-HANDLER*")))))))))
 
 (setq *dispatch-table*
-        (nconc
-         (list (create-static-file-dispatcher-and-handler
-                "/tbnl/test/image.jpg"
-                (make-pathname :name "fz" :type "jpg" :version nil
-                               :defaults (load-time-value *load-pathname*))
-                "image/jpeg"))
-         (mapcar (lambda (args)
-                   (apply #'create-prefix-dispatcher args))
-                 '(("/tbnl/test/form-test.html" form-test)
-                   ("/tbnl/test/forbidden.html" forbidden)
-                   ("/tbnl/test/info.html" info)
-                   ("/tbnl/test/authorization.html" authorization-page)
-                   ("/tbnl/test/image-ram.jpg" image-ram-page)
-                   ("/tbnl/test/cookie.html" cookie-test)
-                   ("/tbnl/test/session.html" session-test)
-                   ("/tbnl/test/parameter_latin1_get.html" parameter-test-latin1-get)
-                   ("/tbnl/test/parameter_latin1_post.html" parameter-test-latin1-post)
-                   ("/tbnl/test/parameter_utf8_get.html" parameter-test-utf8-get)
-                   ("/tbnl/test/parameter_utf8_post.html" parameter-test-utf8-post)
-                   ("/tbnl/test/upload.html" upload-test)
-                   ("/tbnl/test/redir.html" redir)
-                   ("/tbnl/test/oops.html" oops)
-                   #+:lispworks
-                   ("/tbnl/test/utf-8.html" utf-8)
-                   ("/tbnl/test/files/" send-file)
-                   ("/tbnl/test" menu)))
-         (list #'default-dispatcher)))
+      (nconc
+       (list (create-static-file-dispatcher-and-handler
+              "/tbnl/test/image.jpg"
+              (make-pathname :name "fz" :type "jpg" :version nil
+                             :defaults (load-time-value *load-pathname*))
+              "image/jpeg")
+             (create-folder-dispatcher-and-handler
+              "/tbnl/code/"
+              (make-pathname :name nil :type nil :version nil
+                             :defaults (load-time-value *load-pathname*))
+              "text/plain"))
+       (mapcar (lambda (args)
+                 (apply #'create-prefix-dispatcher args))
+               '(("/tbnl/test/form-test.html" form-test)
+                 ("/tbnl/test/forbidden.html" forbidden)
+                 ("/tbnl/test/info.html" info)
+                 ("/tbnl/test/authorization.html" authorization-page)
+                 ("/tbnl/test/image-ram.jpg" image-ram-page)
+                 ("/tbnl/test/cookie.html" cookie-test)
+                 ("/tbnl/test/session.html" session-test)
+                 ("/tbnl/test/parameter_latin1_get.html" parameter-test-latin1-get)
+                 ("/tbnl/test/parameter_latin1_post.html" parameter-test-latin1-post)
+                 ("/tbnl/test/parameter_utf8_get.html" parameter-test-utf8-get)
+                 ("/tbnl/test/parameter_utf8_post.html" parameter-test-utf8-post)
+                 ("/tbnl/test/upload.html" upload-test)
+                 ("/tbnl/test/redir.html" redir)
+                 ("/tbnl/test/oops.html" oops)
+                 #+:lispworks
+                 ("/tbnl/test/utf-8.html" utf-8)
+                 #+:tbnl-bivalent-streams
+                 ("/tbnl/test/umlaut.txt" stream-direct)
+                 ("/tbnl/test/files/" send-file)
+                 ("/tbnl/test" menu)))
+       (list #'default-dispatcher)))
