@@ -1,7 +1,7 @@
 ;;; -*- Mode: LISP; Syntax: COMMON-LISP; Package: TBNL; Base: 10 -*-
-;;; $Header: /usr/local/cvsrep/tbnl/util.lisp,v 1.42 2005/11/18 00:19:27 edi Exp $
+;;; $Header: /usr/local/cvsrep/tbnl/util.lisp,v 1.47 2006/05/24 18:32:43 edi Exp $
 
-;;; Copyright (c) 2004-2005, Dr. Edmund Weitz. All rights reserved.
+;;; Copyright (c) 2004-2006, Dr. Edmund Weitz. All rights reserved.
 
 ;;; Redistribution and use in source and binary forms, with or without
 ;;; modification, are permitted provided that the following conditions
@@ -30,7 +30,8 @@
 (in-package #:tbnl)
 
 #+:lispworks
-(import 'lw:with-unique-names)
+(eval-when (:compile-toplevel :load-toplevel :execute)
+  (import 'lw:with-unique-names))
 
 #-:lispworks
 (defmacro with-unique-names ((&rest bindings) &body body)
@@ -520,6 +521,10 @@ the external format EXTERNAL-FORMAT."
                                                                external-format)
                             do (format s "%~2,'0x" octet)))))))
 
+#+:hunchentoot
+(eval-when (:compile-toplevel :load-toplevel :execute)
+  (require "comm"))
+
 (defun remote-host (socket-stream)
   #-(or :allegro :lispworks) (declare (ignore socket-stream))
   #+:allegro (ignore-errors
@@ -527,3 +532,10 @@ the external format EXTERNAL-FORMAT."
   #+:lispworks (ignore-errors
                  (comm:ip-address-string (comm:socket-stream-peer-address socket-stream))))
 
+(defmacro with-lock-held ((lock) &body body)
+  #-:hunchentoot
+  `(kmrcl::with-lock-held (,lock)
+     ,@body)
+  #+:hunchentoot
+  `(mp:with-lock (,lock)
+     ,@body))
\ No newline at end of file
